// config_signal.h			-*-c++-*-
//
//  A framework for signals that are based on the apt configuration
// code.  This uses a thin wrapper around the apt Configuration class.
//
//  This works in large part because apt-pkg itself does not (thankfully)
// modify configuration items after it is initialized.

#ifndef CONFIG_SIGNAL_H
#define CONFIG_SIGNAL_H

#include <apt-pkg/configuration.h>
#include <sigc++/basic_signal.h>
#include <sigc++/slot.h>

#ifdef HAVE_CONFIG_H
#include "../../config.h"
#endif

#ifdef HAVE_HASH_MAP
#include <hash_map>
#else
#ifdef HAVE_EXT_HASH_MAP
#include <ext/hash_map>
#else
// Fallback to the non-hashing map class
#include <map>
#define hash_map map
#endif
#endif

#include <vector>

#include "strhash.h"

class signalling_config
{
  Configuration *real_config;

  typedef std::hash_map<string, SigC::Signal1<void, string> *> connhash;

  connhash conn_table;
public:
  signalling_config(Configuration *_real_config):real_config(_real_config) {}
  ~signalling_config();

  inline string Find(const char *Name,const char *Default = 0)
  {return real_config->Find(Name, Default);}
  inline string Find(string Name,const char *Default = 0)
  {return real_config->Find(Name, Default);}
  inline string FindFile(const char *Name,const char *Default = 0)
  {return real_config->FindFile(Name, Default);}
  inline string FindDir(const char *Name,const char *Default = 0)
  {return real_config->FindDir(Name, Default);}
  inline int FindI(const char *Name,int Default = 0)
  {return real_config->FindI(Name, Default);}
  inline int FindI(string Name,bool Default = 0)
  {return real_config->FindI(Name, Default);}

  inline bool FindB(const char *Name,bool Default = false)
  {return real_config->FindB(Name, Default);}
  inline bool FindB(string Name,bool Default = false)
  {return real_config->FindB(Name, Default);}

  void Set(string Name,string Value);
  void Set(const char *Name,string Value);
  void Set(const char *Name,int Value);

  Configuration *setcfg(Configuration *newcfg);
  // Switches to a different configuration tree (this will involve emitting
  // various signals)  Returns the old configuration tree.

  inline bool Exists(string Name)
  {return real_config->Exists(Name);}
  bool Exists(const char *Name)
  {return real_config->Exists(Name);}
      
  inline const Configuration::Item *Tree(const char *Name)
  {return real_config->Tree(Name);}

  void connect(string name, const SigC::Slot1<void, string> &slot);
  void connectI(string name, const SigC::Slot1<void, int> &slot);
  void connectB(string name, const SigC::Slot1<void, bool> &slot);

  void Dump()
  {real_config->Dump();}
};

#endif
