// testscr.cc
//
//  Copyright 1999,2000,2001 Daniel Burrows
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; see the file COPYING.  If not, write to
//  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//  Boston, MA 02111-1307, USA.
//
//  Tests the various screen-output mechanisms

#define _GNU_SOURCE // For getopt.h

#include <getopt.h>

#include "aptitude.h"

#include "../config.h"

#include "generic/apt.h"
#include "generic/config_signal.h"

#include <vscreen/config/keybindings.h>
#include <vscreen/vscreen.h>
#include <vscreen/vs_util.h>

#include <sigc++/func_slot.h>

#include <apt-pkg/init.h>

#include "ui.h"

#include "vs_progress.h"
#include "pkg_columnizer.h"
#include "pkg_grouppolicy.h"
#include "pkg_view.h"

#if 0
class mypkgtree:public pkg_tree
{
protected:
  bool quit(bool exit_prog, bool from_user, bool revert)
  {
    if(!from_user || pkg_tree::quit(exit_prog, from_user, revert))
      {
	if(!revert)
	  save_selections();
	return true;
      }
    else
      return false;
  }
public:
  void init_apt()
  {
    vs_progress progbar;
    add_widget(&progbar);
    apt_init(&progbar);
    rem_widget(&progbar);
  }

  void save_selections()
  {
    if( (aptitudeDepCache *) apt_cache_file)
      {
	vs_progress progbar;
	add_widget(&progbar);
	(*apt_cache_file)->save_selection_list(progbar);
	rem_widget(&progbar);
      }
  }
};
#endif

static void do_vscreen_update(const pkgCache::PkgIterator &)
{
  vscreen_update();
}

static void usage()
{
  printf(PACKAGE " " VERSION "\n");
  printf("%s", _("Usage: aptitude [-S fname] [-u|-i]"));
  printf("\n");
  printf("%s", _("  Options:\n"));
  printf("%s",
	 _(" -S fname: read the aptitude extended status info from fname.\n"));
  printf("%s",
	 _(" -u      : Download new package lists on startup.\n"));
  printf("%s",
	 _(" -i      : Perform an install run on startup.\n"));
}

option opts[]={
  {"--help", 0, NULL, 'h'}
};

const char *argv0;

int main(int argc, char *argv[])
{
  char *status_fname=NULL;
  // The file-descriptor to read status information in on, (-1 = nonesuch)

  bool update_only=false, install_only=false;

  int curopt;
  // The last option seen

  // NOTE: this can of course be spoofed.  Anyone bothering to is off their
  //      rocker.
  argv0=argv[0];

  // Read the arguments:
  while((curopt=getopt_long(argc, argv, "hS:ui", opts, NULL))!=-1)
    {
      switch(curopt)
	{
	case 'S':
	  status_fname=strdup(optarg);
	  break;
	case 'h':
	  usage();
	  exit(0);
	case 'u':
	  update_only=true;
	  break;
	case 'i':
	  install_only=true;
	  break;
	case ':':
	case '?':
	  usage();
	  exit(1);
	default:
	  fprintf(stderr, "%s",
		  _("WEIRDNESS: unknown option code received\n"));
	  break;
	}
    }

  // Sanity-check
  if(update_only && install_only)
    {
      fprintf(stderr, "%s",
	      _("Only one of -u and -i may be specified\n"));
      usage();
      exit(1);
    }

  srandom(time(0));

  setlocale(LC_ALL, "");
  bindtextdomain(PACKAGE, LOCALEDIR);
  textdomain(PACKAGE);

  apt_preinit();

  ui_init();

  vs_progress *p=gen_progress_bar();
  apt_init(p, status_fname);
  if(status_fname)
    free(status_fname);
  p->destroy();
  check_apt_errors();

#if 0
  mypkgtree *mainscreen=new mypkgtree;

  mainscreen->set_header("Aptitude " VERSION);

  //vs_multiplex *toplevel=new vs_multiplex(0,0,0,0);
  //vscreen_settoplevel(toplevel);
  //toplevel->add_widget(mainscreen);
  //mainscreen->show();

  //mainscreen->snarf_errs();

  mainscreen->init_apt();

  mainscreen->set_grouping(new pkg_grouppolicy_filter_factory(pkg_missing_filter,new pkg_grouppolicy_status_factory(new pkg_grouppolicy_section_factory(pkg_grouppolicy_section_factory::split_subdir,new pkg_grouppolicy_section_factory(pkg_grouppolicy_section_factory::split_topdir,new pkg_grouppolicy_end_factory())))));

#ifdef WITH_PERIOD_BEEP
  vscreen_addtimeout(new silly_timeout, 1000);
#endif
#endif

  file_quit.connect(SigC::slot(vscreen_exitmain));

  if(apt_cache_file)
    {
      (*apt_cache_file)->package_state_changed.connect(SigC::slot(do_vscreen_update));
      (*apt_cache_file)->package_category_changed.connect(SigC::slot(vscreen_update));
    }

  do_new_package_view();

  if(update_only)
    do_update_lists();
  else if(install_only)
    do_package_run_or_show_preview();
    //install_or_remove_packages();

  ui_main();

  return 0;
}
