// pkg_view.h            -*-c++-*-
//
//  Copyright 2000,2001 Daniel Burrows
//
//  Stuff to manage "package views" and a way to create them.  Package views
// are a collection of information displays about a package laid out (by a
// table in our case).  A package view definition has a spot in itself for a
// "main" widget, which allows configurations to be shared between several
// different screens (unfortunately, it prevents me from, eg, having two
// separate package trees at once :-( )

#ifndef PKG_VIEW_H
#define PKG_VIEW_H

#include "vscreen/config/column_definition.h"
#include "pkg_grouppolicy.h"

#include <list>
#include <string>

class pkg_grouppolicy_factory;
class vscreen_widget;
class package_view_item;

// The available types of items are:
//
//  - Main widget: this is filled by the caller and can be any widget.
//  - Static widget: displays formatted text relating to the selected package
//   (eg: description, priority, etc)
//  - Description widget: displays the description of the selected package
enum package_view_itemtype {PACKAGE_VIEW_MAINWIDGET,
			    PACKAGE_VIEW_STATIC,
			    PACKAGE_VIEW_DESCRIPTION};
// NOTE: PACKAGE_VIEW_DESCRIPTION is a nasty temporary hack; I need a more
// flexible info displayer.
//
// Note to self: is the above really true?


// This is just a big structure that transfers information about the
// structure of a view around.  Not particularly pleasant or efficient,
// but it works.
struct package_view_item
{
  package_view_itemtype type;

  column_definition_list *columns;
  // For static items, a column-format
  std::string columns_cfg;
  // For static items, the name of a configuration setting which will be
  // tracked; the columns will be regenerated when this is modified.
  package_view_item *src;
  // For static items, determines where to get the package/version signals from.
  int row, col, w, h;
  int xopts, yopts;
  // obvious table stuff

  int bg;
  // The item's packground attributes.

  std::string popupdownkey;
  // The key used to toggle the visibility of this item.  ("" for none)

  vscreen_widget *widget;
  // For the internal use of make_package_view (stores the widget this item
  // generated)

  bool visible;
  // True if the widget is visible immediately

  package_view_item(column_definition_list *_columns, std::string _columns_cfg,
		    int _row, int _col, int _w, int _h, int _xopts, int _yopts,
		    int _bg, std::string _popupdownkey, bool _visible)
    :type(PACKAGE_VIEW_STATIC), columns(_columns), columns_cfg(_columns_cfg),
       row(_row), col(_col), w(_w), h(_h), xopts(_xopts), yopts(_yopts),
       bg(_bg), popupdownkey(_popupdownkey), widget(NULL),
     visible(_visible) {}

  package_view_item(int _row, int _col, int _w, int _h, int _xopts, int _yopts,
		    int _bg, std::string _popupdownkey, bool _visible)
    :type(PACKAGE_VIEW_DESCRIPTION), columns(NULL),
       row(_row), col(_col), w(_w), h(_h), xopts(_xopts), yopts(_yopts),
       bg(_bg), popupdownkey(_popupdownkey), widget(NULL),
     visible(_visible) {}

  package_view_item(int _row, int _col, int _w, int _h,
		    int _xopts, int _yopts, int _bg, bool _visible)
    :type(PACKAGE_VIEW_MAINWIDGET), columns(NULL),
     row(_row), col(_col), w(_w), h(_h), xopts(_xopts), yopts(_yopts),
     bg(_bg), widget(NULL), visible(_visible) {}
};

// Generates a package view from the given layout with the given main widget.
// If sig is NULL, nothing will be connected to it.
vscreen_widget *make_package_view(std::list<package_view_item> &format,
				  vscreen_widget *mainwidget,
				  pkg_signal *sig);

#endif
